/* -*- mode: c; c-basic-offset: 4; -*- */
#include "trench.h"
#include <string.h>
#include <stdlib.h>

/*======================================  Identify the channel type  */
void 
trench_init(struct trench_struct* t) {
    t->str   = NULL;
    t->len   = 0;
    t->styp  = trch_base;
    t->ctype = cUnknown;
    t->rate  = 0.0;
    t->dtype = _undefined;
}

/*======================================  Identify the channel type  */
void 
trench_parse(struct trench_struct* t, const char* s) {
    /*----------------------------------  Copy string, find the first ' '    */
    t->str = strdup(s);
    char * pterm = strchr(t->str, ' ');
    if (pterm) *pterm = 0;

    /*----------------------------------  Look for c-type or rate at end     */
    while ((pterm = strrchr(t->str, ',')) != NULL) {
	*pterm++ = 0;
	if (*pterm >= '0' && *pterm <= '9') {
	    t->rate = strtod(pterm, 0);
	} else {
	    t->ctype = cvt_str_chantype(pterm);
	}
    }

    /*----------------------------------  Look for a trend subchannel suffix */
    t->styp = trch_base;
    const char* pdot = strrchr(t->str, '.');
    if (pdot) {
	size_t ldot = strlen(pdot);
	if (ldot == 2 && !strcmp(pdot, ".n")) {
	    t->styp = trch_n;
	} else if (ldot == 4) {
	    if (!strcmp(pdot, ".max")) {
		t->styp = trch_max;
	    } else if (!strcmp(pdot, ".min")) {
		t->styp = trch_min;
	    } else if (!strcmp(pdot, ".rms")) {
		t->styp = trch_rms;
	    }
	} else if (ldot == 5 && !strcmp(pdot, ".mean")) {
	    t->styp = trch_mean;
	}
    }
    if (t->styp == trch_base) t->len = strlen(t->str);
    else                      t->len = pdot - t->str;

    /*----------------------------------  Use inference to fill in data */
    if (t->ctype == cUnknown && t->styp != trch_base) {
	if (t->rate > 0.017) t->ctype = cSTrend;
	else                 t->ctype = cMTrend;
    }

    if (t->ctype != cUnknown && t->styp != trch_base && t->rate == 0) {
	if (t->ctype == cSTrend) t->rate = 1.0; 
	if (t->ctype == cMTrend) t->rate = 1.0/60.0;
    }

    if (t->styp != trch_base && t->dtype == _undefined) {
	t->dtype = trench_dtype(t);
    }
}

/*======================================  Compare the base name  */
int
trench_cmp_base(struct trench_struct* t, const char* s) {
    int rc = -1;
    if (t->styp == trch_base) rc = strcmp (t->str, s);
    else                      rc = strncmp(t->str, s, t->len);
    return rc;
}

/*======================================  Get data type         */
void
trench_destroy(struct trench_struct* t) {
    if (t->str) free(t->str);
    t->str = 0;
}

/*======================================  Get data type         */
daq_data_t
trench_dtype(struct trench_struct* t) {
    daq_data_t dtype;
    switch (t->styp) {
    case trch_mean:
    case trch_rms:
	dtype = _64bit_double;
	break;
    case trch_min:
    case trch_max:
	dtype = _32bit_float;
	break;
    case trch_n:
	dtype = _32bit_integer;
	break;
    default:
	dtype = _undefined;
    }
    return dtype;
}
